/*
  ICFS: Interactively Controlled File System
  Copyright (C) 2024-2025  Fedir Kovalov

  This program can be distributed under the terms of the GNU GPLv2.
  See the file LICENSE.
*/

#ifndef PROC_OPERATIONS
#define PROC_OPERATIONS

#include <time.h>

char *get_process_name_by_pid(const int pid);

/**
 * Finds the parent process ID of a given process.
 *
 * @param pid: The process ID of the process to find the parent of
 * @return: The parent process ID, or 0 if the parent process ID could not be
 * found
 */
pid_t get_parent_pid(pid_t pid);

/**
 * @brief Returns the PID of the main thread (i.e., the process ID) of the
 * process that the given thread ID (tid) belongs to.
 *
 * @param tid The thread ID (TID) of any thread in the process.
 * @return pid_t The process ID (main thread's PID), or -1 on error.
 */
pid_t get_main_thread_pid(pid_t tid);

#endif // !PROC_OPERATIONS
